"""
Goodrich et al
"Data Structures and Algorithms"
Chapter 3: Algorithm Analysis

Problem C-3.35
Assuming it is possible to sort n numbers in O(n log n) time,
show that it is possible to solve the three-way set disjointness problem
in O(n log n) time.

Three way set disjointness problem:
    Given three sequences of nubers, A B and C
    three way set disjointness problem is to determine
    if the intersection of the three sequences is empty
    namely, that there is no element x such that 
    x is in A, B, and C. 
"""
from binary_search import binary_search

def three_way_disjoint(A,B,C):

    # Overall algorithm analysis:
    # O(n log n) to sort 
    # O(n)*O(log n) to do a binary search for each element
    # O(n)*O(log n)*O(log n) for the very last binary search
    # 
    # Overall, this is O(n log n)

    # Built-in merge sort is O(n log n) 
    A = sorted(A)
    B = sorted(B)
    C = sorted(C)

    founddupe = False
    for a in A:
        if(binary_search(a,B)>=0):
            if(binary_search(a,C)>=0):
                founddupe = True
                break 
    
    if(founddupe):
        print("A, B, and C are NOT disjoint: intersection contains at least 1 element.")
    else:
        print("A, B, and C are disjoint. Long live the Oxford comma.")


if __name__=="__main__":

    # Start with the three sets
    A = [z for z in "fug"]
    B = [z for z in "bur"]
    C = [z for z in "wuz"]
    
    three_way_disjoint(A,B,C)

