from __future__ import division
from math import sqrt, floor, ceil

"""
Sieve of Eratosthenes

Constructs a bit vector from 0 to sqrt(n)
and eliminates non-primes from the bit vector.
"""

def sieve(n):
    """
    Perform sieve of eratosthenes and return the resulting bit vector.
    """
    nsqrt = int(floor(sqrt(n)))

    # Do this in the most basic and boring way possible,
    # which also wastes lots of space for even numbers. 
    primes = [True for j in range(nsqrt)]
    primes[0] = False
    primes[1] = False

    # Sieve the non-primes.
    for i in range(2,nsqrt+1):
        for j in range(2,int(ceil(nsqrt/i))):
            # Make sure j starts at 2, otherwise you eliminate every single number.
            # Also make sure you to ceiling of nsqrt/i, so you don't accidentally 
            # skip a non-prime you were supposed to mark as non-prime.
            # Also make sure you are doing float division and not inteer division,
            # or else the ceiling function call will not make a difference 
            # and you will have a non-prime-not-marked-as-non-prime error.
            primes[j*i] = False

    return primes



def print_primes(primes):
    """
    Print the primes bit vector.
    """
    for i in range(len(primes)):
        if(primes[i]):
            print(i)



if __name__=="__main__":
    print_primes(sieve(10000))

